using UnityEngine;


[ExecuteInEditMode]
public class RandomMaterial : MonoBehaviour {

    
    public enum vectorEnum { X, Y, Z, XY, XZ, YZ };
    [Header("Generation")]
    [Tooltip("The transform plane the randomization is based on")]
    public vectorEnum randomizeationVector;
    [Tooltip("The possible materials to randomize between")]
    public Material[] Materials;
    [Header("Manual Control")]
    [Tooltip("Manually control the material")]
    public bool overrideMaterial = false;
    [Tooltip("Material index to use for manual override")]
    public int OverrideIndex = 0;
    [Header("Apply")]
    [Tooltip("This will apply all of the settings and delete the script")]
    public bool ApplyChanges = false;

    private float p = 0f;
    private Material targetMaterial;
    private Renderer rend;


    void OnStart()
    {
        if (Application.isPlaying) // fail safe incase he script is still applied at run time.
        {
            Destroy(this);
        }
    }


    public void Update (){
        
        if (Materials.Length >= 1 && Materials[Materials.Length-1] != null && !Application.isPlaying) // Stops the script from removing the default material until the array has been filled.
        {
            switch (randomizeationVector)
            {
                case vectorEnum.X:
                    p = randomValueFromFloat(this.gameObject.transform.position.x);
                    break;
                case vectorEnum.Y:
                    p = randomValueFromFloat(this.gameObject.transform.position.y);
                    break;
                case vectorEnum.Z:
                    p = randomValueFromFloat(this.gameObject.transform.position.z);
                    break;
                case vectorEnum.XY:
                    p = randomValueFromFloat(this.gameObject.transform.position.x * this.gameObject.transform.position.y);
                    break;
                case vectorEnum.XZ:
                    p = randomValueFromFloat(this.gameObject.transform.position.x * this.gameObject.transform.position.y);
                    break;
                case vectorEnum.YZ:
                    p = randomValueFromFloat(this.gameObject.transform.position.y * this.gameObject.transform.position.z);
                    break;
            }

            for (int x = 0; x < Materials.Length; x++)
            {
                if (applyMaterial(p, Materials.Length, x))
                {
                    targetMaterial = Materials[x];
                    break;
                }
            }
            
           
            if (overrideMaterial)
            {
                if (OverrideIndex >= Materials.Length)
                {
                    OverrideIndex = Materials.Length - 1; // stops the index value going above the array limit.
                }
                targetMaterial = Materials[OverrideIndex];
            }
            rend = this.GetComponent<Renderer>();
            rend.material = targetMaterial;

            if (ApplyChanges)
            {
                DestroyImmediate(this);
            }
        }
	}

    private float randomValueFromFloat(float seed)
    {
        Random.seed = (int)seed;
        return (Random.value);

    }
    

    private bool applyMaterial(float p, int length, int x)
    {
        if (p < (1f / length) + ((float)x / length)){return (true);}
        else { return (false); }

    }

}